ALBUM = "album"
AUDIOBOOK = "audiobook"
BOOK = "book"
CHAPTER = "chapter"
COMIC = "comic"
DOCUMENT = "document"
MOVIE = "movie"
MUSIC = "music"
PODCAST = "podcast"
SPECIAL = "special"
SUBSCRIPTION = "subscription"
VIDEO = "video"


MEDIA_TYPES = [
    ALBUM,
    AUDIOBOOK,
    BOOK,
    CHAPTER,
    COMIC,
    DOCUMENT,
    MOVIE,
    MUSIC,
    PODCAST,
    SPECIAL,
    SUBSCRIPTION,
    VIDEO,
]



from typing import Iterable, List
import os

from sourcesquirrel import excel
from sourcesquirrel.classes.blockchain import Blockchain
from sourcesquirrel.classes.collection import Collection
from sourcesquirrel.classes.media_type import MediaType
from sourcesquirrel.classes.platform import Platform
from sourcesquirrel.classes.release_type import ReleaseType
from sourcesquirrel.classes.series import Series
from sourcesquirrel.integrations.google.drive import GoogleDriveClient
from sourcesquirrel.prefabs import blockchains, media_types, platforms, release_types


class SourceOfTruth:
    def __init__(self, collections: List[Collection], series: List[Series]):
        self.collections: List[Collection] = collections
        self.series: List[Series] = series

    @property
    def blockchains(self) -> Iterable[Blockchain]:
        return list(blockchains.BLOCKCHAINS.values())

    @property
    def media_types(self) -> Iterable[MediaType]:
        return list(media_types.MEDIA_TYPES.values())

    @property
    def platforms(self) -> Iterable[Platform]:
        return list(platforms.PLATFORMS.values())

    @property
    def release_types(self) -> Iterable[ReleaseType]:
        return list(release_types.RELEASE_TYPES.values())

    @staticmethod
    def load_xlsx(path: str) -> "SourceOfTruth":
        series = {s.slug: s for s in excel.load_series(path)}
        collections = [c for c in excel.load_collections(path, series)]

        return SourceOfTruth(collections, list(series.values()))

    @staticmethod
    def load_from_drive(
        google_secrets_json: str,
        file_drive_id: str,
        file_path: str = "/tmp/sot.xlsx",
    ) -> "SourceOfTruth":
        client = GoogleDriveClient.from_google_secrets_json(google_secrets_json)
        client.download(file_drive_id, file_path)

        return SourceOfTruth.load_xlsx(file_path)



from typing import Optional


class Blockchain:
    def __init__(self, name: str, currency: str, coinmarketcap_id: int = None):
        self.name: str = name
        self.currency: str = currency

        self.coinmarketcap_id: Optional[int] = coinmarketcap_id

    def logo_url(self) -> str:  # pragma: no cover
        return f"https://s2.coinmarketcap.com/static/img/coins/64x64/{self.coinmarketcap_id}.png"

    def __str__(self) -> str:  # pragma: no cover
        return f"{self.name} ({self.currency})"



from sourcesquirrel.classes.media_type import MediaType
from sourcesquirrel.constants import emojis as e
from sourcesquirrel.constants import media_types as mt
from sourcesquirrel.prefabs.platforms import BOOK_IO, STUFF_IO

SPECIAL = MediaType(mt.SPECIAL, e.GEM, None)
SUBSCRIPTION = MediaType(mt.SUBSCRIPTION, e.ARROWS, None)

AUDIOBOOK = MediaType(mt.AUDIOBOOK, e.HEADPHONES, BOOK_IO)
BOOK = MediaType(mt.BOOK, e.BOOK, BOOK_IO)
CHAPTER = MediaType(mt.CHAPTER, e.MEMO, BOOK_IO)
COMIC = MediaType(mt.COMIC, e.TEXT_BUBBLE, BOOK_IO)
DOCUMENT = MediaType(mt.DOCUMENT, e.PAGE, BOOK_IO)

ALBUM = MediaType(mt.ALBUM, e.CD, STUFF_IO)
MOVIE = MediaType(mt.MOVIE, e.FILM, STUFF_IO)
MUSIC = MediaType(mt.MUSIC, e.MUSICAL_NOTE, STUFF_IO)
PODCAST = MediaType(mt.PODCAST, e.MICROPHONE, STUFF_IO)
VIDEO = MediaType(mt.VIDEO, e.VCR, STUFF_IO)

MEDIA_TYPES = {
    mt.ALBUM: ALBUM,
    mt.AUDIOBOOK: AUDIOBOOK,
    mt.BOOK: BOOK,
    mt.CHAPTER: CHAPTER,
    mt.COMIC: COMIC,
    mt.DOCUMENT: DOCUMENT,
    mt.MOVIE: MOVIE,
    mt.MUSIC: MUSIC,
    mt.PODCAST: PODCAST,
    mt.SPECIAL: SPECIAL,
    mt.SUBSCRIPTION: SUBSCRIPTION,
    mt.VIDEO: VIDEO,
}


def get(name: str) -> MediaType:
    return MEDIA_TYPES[name]



from sourcesquirrel.classes.blockchain import Blockchain
from sourcesquirrel.constants import blockchains as b
from sourcesquirrel.constants import cryptocurrencies as c

ALGORAND = Blockchain(b.ALGORAND, c.ALGO, 4030)
CARDANO = Blockchain(b.CARDANO, c.ADA, 2010)
COINBASE = Blockchain(b.COINBASE, c.BASE, 27789)
ETHEREUM = Blockchain(b.ETHEREUM, c.ETH, 1027)
POLYGON = Blockchain(b.POLYGON, c.POL, 3890)

BLOCKCHAINS = {
    b.ALGORAND: ALGORAND,
    b.CARDANO: CARDANO,
    b.COINBASE: COINBASE,
    b.ETHEREUM: ETHEREUM,
    b.POLYGON: POLYGON,
}


def get(name: str) -> Blockchain:
    return BLOCKCHAINS[name]



class ReleaseType:
    def __init__(self, name: str, emoji: str):
        self.name: str = name
        self.emoji: str = emoji

    def __str__(self) -> str:  # pragma: no cover
        return f"{self.emoji} {self.name}"



from uuid import UUID

from sourcesquirrel.classes.blockchain import Blockchain


class Series:
    def __init__(self, id: UUID, name: str, slug: str, emoji: str, blockchain: Blockchain):
        self.id: UUID = id
        self.name: str = name
        self.slug: str = slug
        self.emoji: str = emoji
        self.blockchain: Blockchain = blockchain

    def __str__(self) -> str:  # pragma: no cover
        return f"{self.emoji} {self.name} ({self.blockchain.name})"



AIRDROP = "airdrop"
FLASH = "flash"
GIVEAWAY = "giveaway"
MINT = "mint"
SALE = "sale"
SPECIAL = "special"
SUBSCRIPTION = "subscription"


RELEASE_TYPES = [
    AIRDROP,
    FLASH,
    GIVEAWAY,
    MINT,
    SALE,
    SPECIAL,
    SUBSCRIPTION,
]
